@@ -128,11 +128,6 @@ pub fn typescript_prelude() -> Result<()> {
 }
 
 pub fn package_interface(path: Utf8PathBuf) -> Result<()> {
-    // Reset the build directory so we know the state of the project
-    let paths = crate::find_project_paths()?;
-    let config = crate::config::root_config()?;
-    crate::fs::delete_directory(&paths.build_directory_for_target(Mode::Prod, config.target))?;
-
     // Build the project
     let mut built = crate::build::main(
         Options {
@@ -147,6 +142,6 @@ pub fn package_interface(path: Utf8PathBuf) -> Result<()> {
     built.root_package.attach_doc_and_module_comments();
 
     let out = gleam_core::docs::generate_json_package_interface(path, &built.root_package);
-    crate::fs::write_outputs_under(&[out], paths.root())?;
+    crate::fs::write_outputs_under(&[out], crate::find_project_paths()?.root())?;
     Ok(())
 }
