@@ -108,6 +108,7 @@ pub fn command(
             Runtime::NodeJs => {
                 run_javascript_node(&paths, &main_function.package, &module, arguments)
             }
+            Runtime::Bun => run_javascript_bun(&paths, &main_function.package, &module, arguments),
         },
     }?;
 
@@ -130,7 +131,7 @@ fn run_erlang(
         args.push(entry.path().join("ebin").into());
     }
 
-    // gleam modules are seperated by `/`. Erlang modules are separated by `@`.
+    // gleam modules are separated by `/`. Erlang modules are separated by `@`.
     let module = module.replace('/', "@");
 
     args.push("-eval".into());
@@ -148,6 +149,24 @@ fn run_erlang(
     ProjectIO::new().exec("erl", &args, &[], None, Stdio::Inherit)
 }
 
+fn run_javascript_bun(
+    paths: &ProjectPaths,
+    package: &str,
+    module: &str,
+    arguments: Vec<String>,
+) -> Result<i32, Error> {
+    let mut args = vec!["run".to_string()];
+    let entry = write_javascript_entrypoint(paths, package, module)?;
+
+    args.push(entry.to_string());
+
+    for arg in arguments.into_iter() {
+        args.push(arg);
+    }
+
+    ProjectIO::new().exec("bun", &args, &[], None, Stdio::Inherit)
+}
+
 fn run_javascript_node(
     paths: &ProjectPaths,
     package: &str,
@@ -201,6 +220,11 @@ fn run_javascript_deno(
         args.push("--unstable".into())
     }
 
+    // Enable location API
+    if let Some(location) = &config.javascript.deno.location {
+        args.push(format!("--location={}", location));
+    }
+
     // Set deno permissions
     if config.javascript.deno.allow_all {
         // Allow all
