@@ -102,12 +102,14 @@ pub struct PackageConfig {
 impl PackageConfig {
     pub fn dependencies_for(&self, mode: Mode) -> Result<Dependencies> {
         match mode {
-            Mode::Dev | Mode::Lsp => self.all_dependencies(),
+            Mode::Dev | Mode::Lsp => self.all_drect_dependencies(),
             Mode::Prod => Ok(self.dependencies.clone()),
         }
     }
 
-    pub fn all_dependencies(&self) -> Result<Dependencies> {
+    // Return all the dependencies listed in the configuration, that is, all the
+    // direct dependencies, both in the `dependencies` and `dev-dependencies`.
+    pub fn all_drect_dependencies(&self) -> Result<Dependencies> {
         let mut deps =
             HashMap::with_capacity(self.dependencies.len() + self.dev_dependencies.len());
         for (name, requirement) in self.dependencies.iter().chain(&self.dev_dependencies) {
@@ -148,7 +150,7 @@ impl PackageConfig {
         match manifest {
             None => Ok(HashMap::new()),
             Some(manifest) => {
-                StalePackageRemover::fresh_and_locked(&self.all_dependencies()?, manifest)
+                StalePackageRemover::fresh_and_locked(&self.all_drect_dependencies()?, manifest)
             }
         }
     }
@@ -309,7 +311,7 @@ fn locked_no_changes() {
     ]
     .into();
     let manifest = Manifest {
-        requirements: config.all_dependencies().unwrap(),
+        requirements: config.all_drect_dependencies().unwrap(),
         packages: vec![
             manifest_package("prod1", "1.1.0", &[]),
             manifest_package("prod2", "1.2.0", &[]),
@@ -335,7 +337,7 @@ fn locked_some_removed() {
     config.dependencies = [("prod1".into(), Requirement::hex("~> 1.0"))].into();
     config.dev_dependencies = [("dev2".into(), Requirement::hex("~> 2.0"))].into();
     let manifest = Manifest {
-        requirements: config.all_dependencies().unwrap(),
+        requirements: config.all_drect_dependencies().unwrap(),
         packages: vec![
             manifest_package("prod1", "1.1.0", &[]),
             manifest_package("prod2", "1.2.0", &[]), // Not in config
