@@ -113,7 +113,9 @@ pub fn records(module: &TypedModule) -> Vec<(&str, String)> {
                          type_,
                          ..
                      }| {
-                        label.as_deref().map(|label| (label, type_.clone()))
+                        label
+                            .as_ref()
+                            .map(|(_, label)| (label.as_str(), type_.clone()))
                     },
                 )
                 .collect::<Option<Vec<_>>>()
@@ -241,14 +243,19 @@ fn register_imports(
     match s {
         Definition::Function(Function {
             publicity,
-            name,
+            name: Some((_, name)),
             arguments: args,
             implementations,
             ..
         }) if publicity.is_importable() || overridden_publicity.contains(name) => {
             // If the function isn't for this target then don't attempt to export it
             if implementations.supports(Target::Erlang) {
-                exports.push(atom_string(name.to_string()).append("/").append(args.len()))
+                let function_name = escape_erlang_existing_name(name);
+                exports.push(
+                    atom_string(function_name.to_string())
+                        .append("/")
+                        .append(args.len()),
+                )
             }
         }
 
@@ -371,7 +378,13 @@ fn module_function<'a>(
         return None;
     }
 
-    let mut env = Env::new(module, &function.name, line_numbers);
+    let (_, function_name) = function
+        .name
+        .as_ref()
+        .expect("A module's function must be named");
+    let function_name = escape_erlang_existing_name(function_name);
+
+    let mut env = Env::new(module, function_name, line_numbers);
     let var_usages = collect_type_var_usages(
         HashMap::new(),
         std::iter::once(&function.return_type).chain(function.arguments.iter().map(|a| &a.type_)),
@@ -382,17 +395,26 @@ fn module_function<'a>(
         .iter()
         .map(|a| type_printer.print(&a.type_));
     let return_spec = type_printer.print(&function.return_type);
-    let spec = fun_spec(&function.name, args_spec, return_spec);
+    let spec = fun_spec(function_name, args_spec, return_spec);
     let arguments = fun_args(&function.arguments, &mut env);
 
     let body = function
         .external_erlang
         .as_ref()
-        .map(|(module, function)| docvec![atom(module), ":", atom(function), arguments.clone()])
+        .map(|(module, function)| {
+            docvec![
+                atom(module),
+                ":",
+                atom(escape_erlang_existing_name(function)),
+                arguments.clone()
+            ]
+        })
         .unwrap_or_else(|| statement_sequence(&function.body, &mut env));
 
     let doc = spec
-        .append(atom_string(function.name.to_string()))
+        .append(atom_string(
+            escape_erlang_existing_name(function_name).to_string(),
+        ))
         .append(arguments)
         .append(" ->")
         .append(line().append(body).nest(INDENT).group())
@@ -403,7 +425,7 @@ fn module_function<'a>(
 fn fun_args<'a>(args: &'a [TypedArg], env: &mut Env<'a>) -> Document<'a> {
     wrap_args(args.iter().map(|a| match &a.names {
         ArgNames::Discard { .. } | ArgNames::LabelledDiscard { .. } => "_".to_doc(),
-        ArgNames::Named { name } | ArgNames::NamedLabelled { name, .. } => {
+        ArgNames::Named { name, .. } | ArgNames::NamedLabelled { name, .. } => {
             env.next_local_var_name(name)
         }
     }))
@@ -512,6 +534,65 @@ fn tuple<'a>(elems: impl IntoIterator<Item = Document<'a>>) -> Document<'a> {
         .group()
 }
 
+fn const_string_concatenate_bit_array<'a>(
+    elems: impl IntoIterator<Item = Document<'a>>,
+) -> Document<'a> {
+    join(elems, break_(",", ", "))
+        .nest(INDENT)
+        .surround("<<", ">>")
+        .group()
+}
+
+fn const_string_concatenate<'a>(
+    left: &'a TypedConstant,
+    right: &'a TypedConstant,
+    env: &mut Env<'a>,
+) -> Document<'a> {
+    let left = const_string_concatenate_argument(left, env);
+    let right = const_string_concatenate_argument(right, env);
+    const_string_concatenate_bit_array([left, right])
+}
+
+fn const_string_concatenate_inner<'a>(
+    left: &'a TypedConstant,
+    right: &'a TypedConstant,
+    env: &mut Env<'a>,
+) -> Document<'a> {
+    let left = const_string_concatenate_argument(left, env);
+    let right = const_string_concatenate_argument(right, env);
+    join([left, right], break_(",", ", "))
+}
+
+fn const_string_concatenate_argument<'a>(
+    value: &'a TypedConstant,
+    env: &mut Env<'a>,
+) -> Document<'a> {
+    match value {
+        Constant::String { value, .. } => docvec!['"', string_inner(value), "\"/utf8"],
+
+        Constant::Var {
+            constructor: Some(constructor),
+            ..
+        } => match &constructor.variant {
+            ValueConstructorVariant::ModuleConstant {
+                literal: Constant::String { value, .. },
+                ..
+            } => docvec!['"', string_inner(value), "\"/utf8"],
+            ValueConstructorVariant::ModuleConstant {
+                literal: Constant::StringConcatenation { left, right, .. },
+                ..
+            } => const_string_concatenate_inner(left, right, env),
+            _ => const_inline(value, env),
+        },
+
+        Constant::StringConcatenation { left, right, .. } => {
+            const_string_concatenate_inner(left, right, env)
+        }
+
+        _ => const_inline(value, env),
+    }
+}
+
 fn string_concatenate<'a>(
     left: &'a TypedExpr,
     right: &'a TypedExpr,
@@ -986,7 +1067,7 @@ fn var<'a>(name: &'a str, constructor: &'a ValueConstructor, env: &mut Env<'a>)
             arity, ref module, ..
         } if module == env.module => "fun "
             .to_doc()
-            .append(atom(name))
+            .append(atom(escape_erlang_existing_name(name)))
             .append("/")
             .append(*arity),
 
@@ -999,7 +1080,7 @@ fn var<'a>(name: &'a str, constructor: &'a ValueConstructor, env: &mut Env<'a>)
             .to_doc()
             .append(module_name_atom(module))
             .append(":")
-            .append(atom(name))
+            .append(atom(escape_erlang_existing_name(name)))
             .append("/")
             .append(*arity),
     }
@@ -1060,6 +1141,10 @@ fn const_inline<'a>(literal: &'a TypedConstant, env: &mut Env<'a>) -> Document<'
             env,
         ),
 
+        Constant::StringConcatenation { left, right, .. } => {
+            const_string_concatenate(left, right, env)
+        }
+
         Constant::Invalid { .. } => panic!("invalid constants should not reach code generation"),
     }
 }
@@ -1361,6 +1446,7 @@ fn module_fn_with_args<'a>(
     args: Vec<Document<'a>>,
     env: &Env<'a>,
 ) -> Document<'a> {
+    let name = escape_erlang_existing_name(name);
     let args = wrap_args(args);
     if module == env.module {
         atom(name).append(args)
@@ -1431,6 +1517,7 @@ fn docs_args_call<'a>(
             ..
         } => {
             let args = wrap_args(args);
+            let name = escape_erlang_existing_name(name);
             // We use the constructor Fn variant's `module` and function `name`.
             // It would also be valid to use the module and label as in the
             // Gleam code, but using the variant can result in an optimisation
@@ -1904,6 +1991,14 @@ pub fn is_erlang_standard_library_module(name: &str) -> bool {
     )
 }
 
+// Includes the functions that are autogenerated by erlang itself
+pub fn escape_erlang_existing_name(name: &str) -> &str {
+    match name {
+        "module_info" => "moduleInfo",
+        _ => name,
+    }
+}
+
 // A TypeVar can either be rendered as an actual type variable such as `A` or `B`,
 // or it can be rendered as `any()` depending on how many usages it has. If it
 // has only 1 usage it is an `any()` type. If it has more than 1 usage it is a
@@ -1923,6 +2018,34 @@ fn collect_type_var_usages<'a>(
     ids
 }
 
+fn result_type_var_ids(ids: &mut HashMap<u64, u64>, arg_ok: &Type, arg_err: &Type) {
+    let mut ok_ids = HashMap::new();
+    type_var_ids(arg_ok, &mut ok_ids);
+
+    let mut err_ids = HashMap::new();
+    type_var_ids(arg_err, &mut err_ids);
+
+    let mut result_counts = ok_ids;
+    for (id, count) in err_ids {
+        let _ = result_counts
+            .entry(id)
+            .and_modify(|current_count| {
+                if *current_count < count {
+                    *current_count = count;
+                }
+            })
+            .or_insert(count);
+    }
+    for (id, count) in result_counts {
+        let _ = ids
+            .entry(id)
+            .and_modify(|current_count| {
+                *current_count += count;
+            })
+            .or_insert(count);
+    }
+}
+
 fn type_var_ids(type_: &Type, ids: &mut HashMap<u64, u64>) {
     match type_ {
         Type::Var { type_: typ } => match typ.borrow().deref() {
@@ -1932,11 +2055,18 @@ fn type_var_ids(type_: &Type, ids: &mut HashMap<u64, u64>) {
             }
             TypeVar::Link { type_: typ } => type_var_ids(typ, ids),
         },
-        Type::Named { args, .. } => {
-            for arg in args {
-                type_var_ids(arg, ids)
+        Type::Named {
+            args, module, name, ..
+        } => match args[..] {
+            [ref arg_ok, ref arg_err] if is_prelude_module(module) && name == "Result" => {
+                result_type_var_ids(ids, arg_ok, arg_err)
             }
-        }
+            _ => {
+                for arg in args {
+                    type_var_ids(arg, ids)
+                }
+            }
+        },
         Type::Fn { args, retrn } => {
             for arg in args {
                 type_var_ids(arg, ids)
@@ -2068,13 +2198,14 @@ impl<'a> TypePrinter<'a> {
                 let arg0 = self.print(args.first().expect("print_prelude_type list"));
                 "list(".to_doc().append(arg0).append(")")
             }
-            "Result" => {
-                let arg_ok = self.print(args.first().expect("print_prelude_type result ok"));
-                let arg_err = self.print(args.get(1).expect("print_prelude_type result err"));
-                let ok = tuple(["ok".to_doc(), arg_ok]);
-                let error = tuple(["error".to_doc(), arg_err]);
-                docvec![ok, break_(" |", " | "), error].nest(INDENT).group()
-            }
+            "Result" => match args {
+                [arg_ok, arg_err] => {
+                    let ok = tuple(["ok".to_doc(), self.print(arg_ok)]);
+                    let error = tuple(["error".to_doc(), self.print(arg_err)]);
+                    docvec![ok, break_(" |", " | "), error].nest(INDENT).group()
+                }
+                _ => panic!("print_prelude_type result expects ok and err"),
+            },
             // Getting here should mean we either forgot a built-in type or there is a
             // compiler error
             name => panic!("{name} is not a built-in type."),
@@ -2150,6 +2281,11 @@ fn find_referenced_private_functions(
             .iter()
             .for_each(|arg| find_referenced_private_functions(&arg.value, already_found)),
 
+        TypedConstant::StringConcatenation { left, right, .. } => {
+            find_referenced_private_functions(left, already_found);
+            find_referenced_private_functions(right, already_found);
+        }
+
         Constant::Tuple { elements, .. } | Constant::List { elements, .. } => elements
             .iter()
             .for_each(|element| find_referenced_private_functions(element, already_found)),
