@@ -1,5 +1,6 @@
 use crate::{
     format::{Formatter, Intermediate},
+    warning::WarningEmitter,
     Error, Result,
 };
 use camino::Utf8Path;
@@ -7,11 +8,12 @@ use ecow::EcoString;
 
 pub fn parse_fix_and_format(src: &EcoString, path: &Utf8Path) -> Result<String> {
     // Parse
-    let parsed = crate::parse::parse_module(src).map_err(|error| Error::Parse {
-        path: path.to_path_buf(),
-        src: src.clone(),
-        error,
-    })?;
+    let parsed = crate::parse::parse_module(path.to_owned(), src, &WarningEmitter::null())
+        .map_err(|error| Error::Parse {
+            path: path.to_path_buf(),
+            src: src.clone(),
+            error,
+        })?;
     let intermediate = Intermediate::from_extra(&parsed.extra, src);
     let module = parsed.module;
 
