@@ -1,3 +1,4 @@
+mod endianness;
 mod expression;
 mod import;
 mod pattern;
@@ -167,7 +168,7 @@ impl<'a> Generator<'a> {
         };
 
         if self.tracker.float_bit_array_segment_used {
-            self.register_prelude_usage(&mut imports, "float64Bits", None);
+            self.register_prelude_usage(&mut imports, "sizedFloat", None);
         };
 
         // Put it all together
@@ -269,7 +270,7 @@ impl<'a> Generator<'a> {
         fn parameter((i, arg): (usize, &TypedRecordConstructorArg)) -> Document<'_> {
             arg.label
                 .as_ref()
-                .map(|s| maybe_escape_identifier_doc(s))
+                .map(|(_, s)| maybe_escape_identifier_doc(s))
                 .unwrap_or_else(|| Document::String(format!("x{i}")))
         }
 
@@ -294,7 +295,7 @@ impl<'a> Generator<'a> {
                 let var = parameter((i, arg));
                 match &arg.label {
                     None => docvec!["this[", i, "] = ", var, ";"],
-                    Some(name) => docvec!["this.", name, " = ", var, ";"],
+                    Some((_, name)) => docvec!["this.", name, " = ", var, ";"],
                 }
             }),
             line(),
@@ -350,7 +351,7 @@ impl<'a> Generator<'a> {
                 }
 
                 Definition::Function(Function {
-                    name,
+                    name: Some((_, name)),
                     publicity,
                     external_javascript: Some((module, function)),
                     ..
@@ -483,6 +484,10 @@ impl<'a> Generator<'a> {
     }
 
     fn module_function(&mut self, function: &'a TypedFunction) -> Option<Output<'a>> {
+        let (_, name) = function
+            .name
+            .as_ref()
+            .expect("A module's function must be named");
         let argument_names = function
             .arguments
             .iter()
@@ -491,7 +496,7 @@ impl<'a> Generator<'a> {
         let mut generator = expression::Generator::new(
             self.module.name.clone(),
             self.line_numbers,
-            function.name.clone(),
+            name.clone(),
             argument_names,
             &mut self.tracker,
             self.module_scope.clone(),
@@ -519,7 +524,7 @@ impl<'a> Generator<'a> {
 
         let document = docvec![
             head,
-            maybe_escape_identifier_doc(function.name.as_str()),
+            maybe_escape_identifier_doc(name.as_str()),
             fun_args(function.arguments.as_slice(), generator.tail_recursion_used),
             " {",
             docvec![line(), body].nest(INDENT).group(),
@@ -532,8 +537,15 @@ impl<'a> Generator<'a> {
     fn register_module_definitions_in_scope(&mut self) {
         for statement in self.module.definitions.iter() {
             match statement {
-                Definition::ModuleConstant(ModuleConstant { name, .. })
-                | Definition::Function(Function { name, .. }) => self.register_in_scope(name),
+                Definition::ModuleConstant(ModuleConstant { name, .. }) => {
+                    self.register_in_scope(name)
+                }
+
+                Definition::Function(Function { name, .. }) => self.register_in_scope(
+                    name.as_ref()
+                        .map(|(_, name)| name)
+                        .expect("Function in a definition must be named"),
+                ),
 
                 Definition::Import(Import {
                     unqualified_values: unqualified,
@@ -768,3 +780,10 @@ pub(crate) struct UsageTracker {
     pub codepoint_bit_array_segment_used: bool,
     pub float_bit_array_segment_used: bool,
 }
+
+fn bool(bool: bool) -> Document<'static> {
+    match bool {
+        true => "true".to_doc(),
+        false => "false".to_doc(),
+    }
+}
