@@ -1,4 +1,7 @@
+use std::sync::Arc;
+
 use ecow::EcoString;
+use itertools::Itertools;
 use lsp_types::{
     CompletionItem, CompletionItemKind, CompletionItemLabelDetails, CompletionTextEdit,
     Documentation, MarkupContent, MarkupKind, Position, Range, TextDocumentPositionParams,
@@ -7,12 +10,13 @@ use lsp_types::{
 use strum::IntoEnumIterator;
 
 use crate::{
-    ast::{Definition, Import, Publicity, TypedDefinition},
+    ast::{CallArg, Definition, Import, Publicity, TypedDefinition, TypedExpr},
     build::Module,
     io::{CommandExecutor, FileSystemReader, FileSystemWriter},
     line_numbers::LineNumbers,
     type_::{
-        pretty::Printer, ModuleInterface, PreludeType, TypeConstructor, ValueConstructorVariant,
+        collapse_links, pretty::Printer, AccessorsMap, FieldMap, ModuleInterface, PreludeType,
+        Type, TypeConstructor, ValueConstructorVariant,
     },
     Result,
 };
@@ -21,6 +25,38 @@ use super::{
     compiler::LspProjectCompiler, files::FileSystemProxy, DownloadDependencies, MakeLocker,
 };
 
+// Represents the kind/specificity of completion that is being requested.
+#[derive(Copy, Clone)]
+enum CompletionKind {
+    // A label for a function or type definition
+    Label,
+    // A field of a record
+    FieldAccessor,
+    // Values or types defined in the current module
+    LocallyDefined,
+    // Values or types defined in an already imported module
+    ImportedModule,
+    // Types defined in the prelude
+    Prelude,
+    // Types defined in a module that has not been imported
+    ImportableModule,
+}
+
+// Gives the sort text for a completion item based on the kind and label.
+// This ensures that more specific kinds of completions are placed before
+// less specific ones..
+fn sort_text(kind: CompletionKind, label: &str) -> String {
+    let priority: u8 = match kind {
+        CompletionKind::Label => 0,
+        CompletionKind::FieldAccessor => 1,
+        CompletionKind::LocallyDefined => 2,
+        CompletionKind::ImportedModule => 3,
+        CompletionKind::Prelude => 4,
+        CompletionKind::ImportableModule => 5,
+    };
+    format!("{}_{}", priority, label)
+}
+
 // The form in which a type completion is needed in context.
 // Mainly used to determine if the "type" keyword should be appended to the completion
 enum TypeCompletionForm {
@@ -29,11 +65,16 @@ enum TypeCompletionForm {
     Default,
 }
 
+enum Newlines {
+    Single,
+    Double,
+}
+
 pub struct Completer<'a, IO> {
     /// The direct buffer source code
     src: &'a EcoString,
     /// The line number information of the buffer source code
-    src_line_numbers: LineNumbers,
+    pub src_line_numbers: LineNumbers,
     /// The current cursor position within the buffer source code
     cursor_position: &'a Position,
     /// A reference to the lsp compiler for getting module information
@@ -229,6 +270,7 @@ where
                 type_,
                 insert_range,
                 TypeCompletionForm::UnqualifiedImport,
+                CompletionKind::ImportedModule,
             ));
         }
 
@@ -252,6 +294,7 @@ where
                 name,
                 value,
                 insert_range,
+                CompletionKind::ImportedModule,
             ));
         }
 
@@ -341,10 +384,13 @@ where
 
         // Prelude types
         for type_ in PreludeType::iter() {
+            let label: String = type_.name().into();
+            let sort_text = Some(sort_text(CompletionKind::Prelude, &label));
             completions.push(CompletionItem {
-                label: type_.name().into(),
+                label,
                 detail: Some("Type".into()),
                 kind: Some(CompletionItemKind::CLASS),
+                sort_text,
                 ..Default::default()
             });
         }
@@ -360,6 +406,7 @@ where
                     type_,
                     insert_range,
                     TypeCompletionForm::Default,
+                    CompletionKind::LocallyDefined,
                 ));
             }
         }
@@ -392,6 +439,7 @@ where
                         type_,
                         insert_range,
                         TypeCompletionForm::Default,
+                        CompletionKind::ImportedModule,
                     ));
                 }
             }
@@ -408,6 +456,7 @@ where
                             type_,
                             insert_range,
                             TypeCompletionForm::Default,
+                            CompletionKind::ImportedModule,
                         )),
                         None => continue,
                     }
@@ -416,7 +465,9 @@ where
         }
 
         // Importable modules
-        let import_location = self.first_import_line_in_module();
+        let (first_import_pos, first_is_import) = self.first_import_in_module();
+        let after_import_newlines =
+            self.add_newlines_after_import(first_import_pos, first_is_import);
         for (module_full_name, module) in self.completable_modules_for_import() {
             // Do not try to import the prelude.
             if module_full_name == "gleam" {
@@ -448,8 +499,14 @@ where
                     type_,
                     insert_range,
                     TypeCompletionForm::Default,
+                    CompletionKind::ImportableModule,
+                );
+                add_import_to_completion(
+                    &mut completion,
+                    first_import_pos,
+                    module_full_name,
+                    &after_import_newlines,
                 );
-                add_import_to_completion(&mut completion, import_location, module_full_name);
                 completions.push(completion);
             }
         }
@@ -473,7 +530,14 @@ where
                 // Here we do not check for the internal attribute: we always want
                 // to show autocompletions for values defined in the same module,
                 // even if those are internal.
-                completions.push(value_completion(None, mod_name, name, value, insert_range));
+                completions.push(value_completion(
+                    None,
+                    mod_name,
+                    name,
+                    value,
+                    insert_range,
+                    CompletionKind::LocallyDefined,
+                ));
             }
         }
 
@@ -505,6 +569,7 @@ where
                         name,
                         value,
                         insert_range,
+                        CompletionKind::ImportedModule,
                     ));
                 }
             }
@@ -523,6 +588,7 @@ where
                                 name,
                                 value,
                                 insert_range,
+                                CompletionKind::ImportedModule,
                             ))
                         }
                         None => continue,
@@ -532,7 +598,9 @@ where
         }
 
         // Importable modules
-        let import_location = self.first_import_line_in_module();
+        let (first_import_pos, first_is_import) = self.first_import_in_module();
+        let after_import_newlines =
+            self.add_newlines_after_import(first_import_pos, first_is_import);
         for (module_full_name, module) in self.completable_modules_for_import() {
             // Do not try to import the prelude.
             if module_full_name == "gleam" {
@@ -557,10 +625,21 @@ where
                     continue;
                 }
 
-                let mut completion =
-                    value_completion(Some(qualifier), module_full_name, name, value, insert_range);
+                let mut completion = value_completion(
+                    Some(qualifier),
+                    module_full_name,
+                    name,
+                    value,
+                    insert_range,
+                    CompletionKind::ImportableModule,
+                );
 
-                add_import_to_completion(&mut completion, import_location, module_full_name);
+                add_import_to_completion(
+                    &mut completion,
+                    first_import_pos,
+                    module_full_name,
+                    &after_import_newlines,
+                );
                 completions.push(completion);
             }
         }
@@ -568,6 +647,96 @@ where
         completions
     }
 
+    // Looks up the type accessors for the given type
+    fn type_accessors_from_modules(
+        &'a self,
+        importable_modules: &'a im::HashMap<EcoString, ModuleInterface>,
+        type_: Arc<Type>,
+    ) -> Option<&AccessorsMap> {
+        let type_ = collapse_links(type_);
+        match type_.as_ref() {
+            Type::Named { name, module, .. } => importable_modules
+                .get(module)
+                .and_then(|i| i.accessors.get(name))
+                .filter(|a| a.publicity.is_importable() || module == &self.module.name),
+            _ => None,
+        }
+    }
+
+    /// Provides completions for field accessors when the context being editted
+    /// is a custom type instance
+    pub fn completion_field_accessors(&'a self, typ: Arc<Type>) -> Vec<CompletionItem> {
+        self.type_accessors_from_modules(
+            self.compiler.project_compiler.get_importable_modules(),
+            typ,
+        )
+        .map(|accessors_map| {
+            accessors_map
+                .accessors
+                .values()
+                .map(|accessor| field_completion(&accessor.label, accessor.type_.clone()))
+                .collect_vec()
+        })
+        .unwrap_or_default()
+    }
+
+    fn callable_field_map(
+        &'a self,
+        expr: &'a TypedExpr,
+        importable_modules: &'a im::HashMap<EcoString, ModuleInterface>,
+    ) -> Option<&'a FieldMap> {
+        match expr {
+            TypedExpr::Var { constructor, .. } => constructor.field_map(),
+            TypedExpr::ModuleSelect {
+                module_name, label, ..
+            } => importable_modules
+                .get(module_name)
+                .and_then(|i| i.values.get(label))
+                .and_then(|a| a.field_map()),
+            _ => None,
+        }
+    }
+
+    /// Provides completions for labels when the context being editted is a call
+    /// that has labelled arguments that can be passed
+    pub fn completion_labels(
+        &'a self,
+        fun: &TypedExpr,
+        existing_args: &[CallArg<TypedExpr>],
+    ) -> Vec<CompletionItem> {
+        let fun_type = fun.type_().fn_types().map(|(args, _)| args);
+        let already_included_labels = existing_args
+            .iter()
+            .filter_map(|a| a.label.clone())
+            .collect_vec();
+        let Some(field_map) =
+            self.callable_field_map(fun, self.compiler.project_compiler.get_importable_modules())
+        else {
+            return vec![];
+        };
+
+        field_map
+            .fields
+            .iter()
+            .filter(|field| !already_included_labels.contains(field.0))
+            .map(|(label, arg_index)| {
+                let detail = fun_type.as_ref().and_then(|args| {
+                    args.get(*arg_index as usize)
+                        .map(|a| Printer::new().pretty_print(a, 0))
+                });
+                let label = format!("{label}:");
+                let sort_text = Some(sort_text(CompletionKind::Label, &label));
+                CompletionItem {
+                    label,
+                    detail,
+                    kind: Some(CompletionItemKind::FIELD),
+                    sort_text,
+                    ..Default::default()
+                }
+            })
+            .collect()
+    }
+
     fn root_package_name(&self) -> &str {
         self.compiler.project_compiler.config.name.as_str()
     }
@@ -587,17 +756,45 @@ where
         }
     }
 
-    // Gets the position of the line with the first import statement in the file.
-    fn first_import_line_in_module(&'a self) -> Position {
-        let import_location = self
+    // Gets the position of the import statement if it's the first definition in the module.
+    // If the 1st definition is not an import statement, then it returns the 1st line.
+    // 2nd element in the pair is true if the first definition is an import statement.
+    fn first_import_in_module(&'a self) -> (Position, bool) {
+        // As "self.module.ast.definitions"  could be sorted, let's find the actual first definition by position.
+        let first_definition = self
             .module
             .ast
             .definitions
             .iter()
-            .find_map(get_import)
-            .map_or(0, |i| i.location.start);
-        let import_location = self.module_line_numbers.line_number(import_location);
-        Position::new(import_location - 1, 0)
+            .min_by(|a, b| a.location().start.cmp(&b.location().start));
+        let import = first_definition.and_then(get_import);
+        let import_start = import.map_or(0, |i| i.location.start);
+        let import_line = self.module_line_numbers.line_number(import_start);
+        (Position::new(import_line - 1, 0), import.is_some())
+    }
+
+    // Returns how many newlines should be added after an import statement. By default `Newlines::Single`,
+    // but if there's not any import statement, it returns `Newlines::Double`.
+    //
+    // * ``import_location`` - The position of the first import statement in the source code.
+    fn add_newlines_after_import(
+        &'a self,
+        import_location: Position,
+        has_imports: bool,
+    ) -> Newlines {
+        let import_start_cursor = self
+            .src_line_numbers
+            .byte_index(import_location.line, import_location.character);
+        let is_new_line = self
+            .src
+            .chars()
+            .nth(import_start_cursor as usize)
+            .unwrap_or_default()
+            == '\n';
+        match !has_imports && !is_new_line {
+            true => Newlines::Double,
+            false => Newlines::Single,
+        }
     }
 }
 
@@ -605,13 +802,18 @@ fn add_import_to_completion(
     item: &mut CompletionItem,
     import_location: Position,
     module_full_name: &EcoString,
+    insert_newlines: &Newlines,
 ) {
+    let new_lines = match insert_newlines {
+        Newlines::Single => "\n",
+        Newlines::Double => "\n\n",
+    };
     item.additional_text_edits = Some(vec![TextEdit {
         range: Range {
             start: import_location,
             end: import_location,
         },
-        new_text: ["import ", module_full_name, "\n"].concat(),
+        new_text: ["import ", module_full_name, new_lines].concat(),
     }]);
 }
 
@@ -621,6 +823,7 @@ fn type_completion(
     type_: &TypeConstructor,
     insert_range: Range,
     include_type_in_completion: TypeCompletionForm,
+    priority: CompletionKind,
 ) -> CompletionItem {
     let label = match module {
         Some(module) => format!("{module}.{name}"),
@@ -637,6 +840,7 @@ fn type_completion(
         label: label.clone(),
         kind,
         detail: Some("Type".into()),
+        sort_text: Some(sort_text(priority, &label)),
         text_edit: Some(CompletionTextEdit::Edit(TextEdit {
             range: insert_range,
             new_text: match include_type_in_completion {
@@ -654,6 +858,7 @@ fn value_completion(
     name: &str,
     value: &crate::type_::ValueConstructor,
     insert_range: Range,
+    priority: CompletionKind,
 ) -> CompletionItem {
     let label = match module_qualifier {
         Some(module) => format!("{module}.{name}"),
@@ -687,6 +892,7 @@ fn value_completion(
             description: Some(module_name.into()),
         }),
         documentation,
+        sort_text: Some(sort_text(priority, &label)),
         text_edit: Some(CompletionTextEdit::Edit(TextEdit {
             range: insert_range,
             new_text: label.clone(),
@@ -695,6 +901,18 @@ fn value_completion(
     }
 }
 
+fn field_completion(label: &str, type_: Arc<Type>) -> CompletionItem {
+    let type_ = Printer::new().pretty_print(&type_, 0);
+
+    CompletionItem {
+        label: label.into(),
+        kind: Some(CompletionItemKind::FIELD),
+        detail: Some(type_),
+        sort_text: Some(sort_text(CompletionKind::FieldAccessor, label)),
+        ..Default::default()
+    }
+}
+
 fn get_import(statement: &TypedDefinition) -> Option<&Import<EcoString>> {
     match statement {
         Definition::Import(import) => Some(import),
