@@ -40,7 +40,6 @@ impl<'a> ModuleEncoder<'a> {
         let mut module = message.init_root::<module::Builder<'_>>();
         module.set_name(&self.data.name);
         module.set_package(&self.data.package);
-        module.set_contains_todo(self.data.contains_todo);
         module.set_src_path(self.data.src_path.as_str());
         module.set_is_internal(self.data.is_internal);
         self.set_module_types(&mut module);
@@ -393,6 +392,12 @@ impl<'a> ModuleEncoder<'a> {
                 );
             }
 
+            Constant::StringConcatenation { right, left, .. } => {
+                let mut builder = builder.init_string_concatenation();
+                self.build_constant(builder.reborrow().init_right(), right);
+                self.build_constant(builder.reborrow().init_left(), left);
+            }
+
             Constant::Invalid { .. } => {
                 panic!("invalid constants should not reach code generation")
             }
