@@ -30,8 +30,8 @@ fn roundtrip(input: &ModuleInterface) -> ModuleInterface {
 
 fn constant_module(constant: TypedConstant) -> ModuleInterface {
     ModuleInterface {
+        warnings: vec![],
         is_internal: true,
-        contains_todo: false,
         package: "some_package".into(),
         origin: Origin::Src,
         name: "a".into(),
@@ -84,8 +84,8 @@ fn bit_array_segment_option_module(option: TypedConstantBitArraySegmentOption) -
 #[test]
 fn empty_module() {
     let module = ModuleInterface {
+        warnings: vec![],
         is_internal: true,
-        contains_todo: false,
         package: "some_package".into(),
         origin: Origin::Src,
         name: "one/two".into(),
@@ -103,8 +103,8 @@ fn empty_module() {
 #[test]
 fn with_line_numbers() {
     let module = ModuleInterface {
+        warnings: vec![],
         is_internal: false,
-        contains_todo: false,
         package: "some_package".into(),
         origin: Origin::Src,
         name: "one/two".into(),
@@ -126,8 +126,8 @@ fn with_line_numbers() {
 #[test]
 fn module_with_private_type() {
     let module = ModuleInterface {
+        warnings: vec![],
         is_internal: false,
-        contains_todo: false,
         package: "some_package".into(),
         origin: Origin::Src,
         name: "a/b".into(),
@@ -157,8 +157,8 @@ fn module_with_private_type() {
 #[test]
 fn module_with_unused_import() {
     let module = ModuleInterface {
+        warnings: vec![],
         is_internal: false,
-        contains_todo: false,
         package: "some_package".into(),
         origin: Origin::Src,
         name: "a".into(),
@@ -179,8 +179,8 @@ fn module_with_unused_import() {
 #[test]
 fn module_with_app_type() {
     let module = ModuleInterface {
+        warnings: vec![],
         is_internal: false,
-        contains_todo: false,
         package: "some_package".into(),
         origin: Origin::Src,
         name: "a/b".into(),
@@ -210,8 +210,8 @@ fn module_with_app_type() {
 #[test]
 fn module_with_fn_type() {
     let module = ModuleInterface {
+        warnings: vec![],
         is_internal: false,
-        contains_todo: false,
         package: "some_package".into(),
         origin: Origin::Src,
         name: "a/b".into(),
@@ -241,8 +241,8 @@ fn module_with_fn_type() {
 #[test]
 fn module_with_tuple_type() {
     let module = ModuleInterface {
+        warnings: vec![],
         is_internal: false,
-        contains_todo: false,
         package: "some_package".into(),
         origin: Origin::Src,
         name: "a/b".into(),
@@ -278,8 +278,8 @@ fn module_with_generic_type() {
 
     fn make(t1: Arc<Type>, t2: Arc<Type>) -> ModuleInterface {
         ModuleInterface {
+            warnings: vec![],
             is_internal: false,
-            contains_todo: false,
             package: "some_package".into(),
             origin: Origin::Src,
             name: "a/b".into(),
@@ -315,8 +315,8 @@ fn module_with_type_links() {
 
     fn make(type_: Arc<Type>) -> ModuleInterface {
         ModuleInterface {
+            warnings: vec![],
             is_internal: false,
-            contains_todo: false,
             package: "some_package".into(),
             origin: Origin::Src,
             name: "a".into(),
@@ -352,8 +352,8 @@ fn module_with_type_constructor_documentation() {
 
     fn make(type_: Arc<Type>) -> ModuleInterface {
         ModuleInterface {
+            warnings: vec![],
             is_internal: false,
-            contains_todo: false,
             package: "some_package".into(),
             origin: Origin::Src,
             name: "a".into(),
@@ -389,8 +389,8 @@ fn module_with_type_constructor_origin() {
 
     fn make(type_: Arc<Type>) -> ModuleInterface {
         ModuleInterface {
+            warnings: vec![],
             is_internal: false,
-            contains_todo: false,
             package: "some_package".into(),
             origin: Origin::Src,
             name: "a".into(),
@@ -425,8 +425,8 @@ fn module_with_type_constructor_origin() {
 #[test]
 fn module_type_to_constructors_mapping() {
     let module = ModuleInterface {
+        warnings: vec![],
         is_internal: false,
-        contains_todo: false,
         package: "some_package".into(),
         origin: Origin::Src,
         name: "a".into(),
@@ -455,8 +455,8 @@ fn module_type_to_constructors_mapping() {
 #[test]
 fn module_fn_value() {
     let module = ModuleInterface {
+        warnings: vec![],
         is_internal: false,
-        contains_todo: false,
         package: "some_package".into(),
         origin: Origin::Src,
         name: "a".into(),
@@ -500,8 +500,8 @@ fn module_fn_value() {
 #[test]
 fn deprecated_module_fn_value() {
     let module = ModuleInterface {
+        warnings: vec![],
         is_internal: false,
-        contains_todo: false,
         package: "some_package".into(),
         origin: Origin::Src,
         name: "a".into(),
@@ -547,8 +547,8 @@ fn deprecated_module_fn_value() {
 #[test]
 fn private_module_fn_value() {
     let module = ModuleInterface {
+        warnings: vec![],
         is_internal: false,
-        contains_todo: false,
         package: "some_package".into(),
         origin: Origin::Src,
         name: "a".into(),
@@ -594,8 +594,8 @@ fn private_module_fn_value() {
 #[test]
 fn module_fn_value_regression() {
     let module = ModuleInterface {
+        warnings: vec![],
         is_internal: false,
-        contains_todo: false,
         package: "some_package".into(),
         origin: Origin::Src,
         name: "a/b/c".into(),
@@ -640,8 +640,8 @@ fn module_fn_value_regression() {
 #[test]
 fn module_fn_value_with_field_map() {
     let module = ModuleInterface {
+        warnings: vec![],
         is_internal: false,
-        contains_todo: false,
         package: "some_package".into(),
         origin: Origin::Src,
         name: "a".into(),
@@ -688,8 +688,8 @@ fn record_value() {
     let mut random = rand::thread_rng();
 
     let module = ModuleInterface {
+        warnings: vec![],
         is_internal: false,
-        contains_todo: false,
         package: "some_package".into(),
         origin: Origin::Src,
         name: "a".into(),
@@ -731,8 +731,8 @@ fn record_value_with_field_map() {
     let mut random = rand::thread_rng();
 
     let module = ModuleInterface {
+        warnings: vec![],
         is_internal: false,
-        contains_todo: false,
         package: "some_package".into(),
         origin: Origin::Src,
         name: "a".into(),
@@ -775,8 +775,8 @@ fn record_value_with_field_map() {
 #[test]
 fn accessors() {
     let module = ModuleInterface {
+        warnings: vec![],
         is_internal: false,
-        contains_todo: false,
         package: "some_package".into(),
         origin: Origin::Src,
         name: "a".into(),
@@ -930,7 +930,7 @@ fn constant_record() {
         name: "".into(),
         args: vec![
             CallArg {
-                implicit: false,
+                implicit: None,
                 label: None,
                 location: Default::default(),
                 value: Constant::Float {
@@ -939,7 +939,7 @@ fn constant_record() {
                 },
             },
             CallArg {
-                implicit: false,
+                implicit: None,
                 label: None,
                 location: Default::default(),
                 value: Constant::Int {
@@ -989,8 +989,8 @@ fn constant_var() {
     };
 
     let module = ModuleInterface {
+        warnings: vec![],
         is_internal: false,
-        contains_todo: false,
         package: "some_package".into(),
         origin: Origin::Src,
         name: "a".into(),
@@ -1209,8 +1209,8 @@ fn constant_bit_array_native() {
 #[test]
 fn deprecated_type() {
     let module = ModuleInterface {
+        warnings: vec![],
         is_internal: false,
-        contains_todo: false,
         package: "some_package".into(),
         origin: Origin::Src,
         name: "a/b".into(),
@@ -1239,31 +1239,11 @@ fn deprecated_type() {
     assert_eq!(roundtrip(&module), module);
 }
 
-#[test]
-fn contains_todo() {
-    let module = ModuleInterface {
-        contains_todo: true,
-        //             ^^^^ It does, it does!
-        is_internal: false,
-        package: "some_package".into(),
-        origin: Origin::Src,
-        name: "a/b".into(),
-        types: [].into(),
-        types_value_constructors: HashMap::new(),
-        values: HashMap::new(),
-        unused_imports: Vec::new(),
-        accessors: HashMap::new(),
-        line_numbers: LineNumbers::new(""),
-        src_path: "some_path".into(),
-    };
-    assert_eq!(roundtrip(&module), module);
-}
-
 #[test]
 fn module_fn_value_with_external_implementations() {
     let module = ModuleInterface {
+        warnings: vec![],
         is_internal: false,
-        contains_todo: false,
         package: "some_package".into(),
         origin: Origin::Src,
         name: "a/b/c".into(),
@@ -1308,8 +1288,8 @@ fn module_fn_value_with_external_implementations() {
 #[test]
 fn internal_module_fn() {
     let module = ModuleInterface {
+        warnings: vec![],
         is_internal: false,
-        contains_todo: false,
         package: "some_package".into(),
         origin: Origin::Src,
         name: "a/b/c".into(),
@@ -1355,8 +1335,8 @@ fn internal_module_fn() {
 #[test]
 fn type_variable_ids_in_constructors_are_shared() {
     let module = ModuleInterface {
+        warnings: vec![],
         is_internal: false,
-        contains_todo: false,
         package: "some_package".into(),
         origin: Origin::Src,
         name: "a/b/c".into(),
