@@ -392,6 +392,7 @@ impl ModuleInterface {
                     typed_parameters,
                     parameters: _,
                     location: _,
+                    name_location: _,
                     end_position: _,
                 }) => {
                     let mut id_map = IdMap::new();
@@ -406,7 +407,7 @@ impl ModuleInterface {
                     let _ = types.insert(
                         name.clone(),
                         TypeDefinitionInterface {
-                            documentation: documentation.clone(),
+                            documentation: documentation.as_ref().map(|(_, doc)| doc.clone()),
                             deprecation: DeprecationInterface::from_deprecation(deprecation),
                             parameters: typed_parameters.len(),
                             constructors: if *opaque {
@@ -415,13 +416,19 @@ impl ModuleInterface {
                                 constructors
                                     .iter()
                                     .map(|constructor| TypeConstructorInterface {
-                                        documentation: constructor.documentation.clone(),
+                                        documentation: constructor
+                                            .documentation
+                                            .as_ref()
+                                            .map(|(_, doc)| doc.clone()),
                                         name: constructor.name.clone(),
                                         parameters: constructor
                                             .arguments
                                             .iter()
                                             .map(|arg| ParameterInterface {
-                                                label: arg.label.clone(),
+                                                label: arg
+                                                    .label
+                                                    .as_ref()
+                                                    .map(|(_, label)| label.clone()),
                                                 // We share the same id_map between each step so that the
                                                 // incremental ids assigned are consisten with each other
                                                 type_: from_type_helper(&arg.type_, &mut id_map),
@@ -443,12 +450,13 @@ impl ModuleInterface {
                     documentation,
                     deprecation,
                     location: _,
+                    name_location: _,
                     type_ast: _,
                 }) => {
                     let _ = type_aliases.insert(
                         alias.clone(),
                         TypeAliasInterface {
-                            documentation: documentation.clone(),
+                            documentation: documentation.as_ref().map(|(_, doc)| doc.clone()),
                             deprecation: DeprecationInterface::from_deprecation(deprecation),
                             parameters: parameters.len(),
                             alias: TypeInterface::from_type(type_.as_ref()),
@@ -465,6 +473,7 @@ impl ModuleInterface {
                     implementations,
                     deprecation,
                     location: _,
+                    name_location: _,
                     annotation: _,
                     value: _,
                 }) => {
@@ -476,7 +485,7 @@ impl ModuleInterface {
                             ),
                             type_: TypeInterface::from_type(type_.as_ref()),
                             deprecation: DeprecationInterface::from_deprecation(deprecation),
-                            documentation: documentation.clone(),
+                            documentation: documentation.as_ref().map(|(_, doc)| doc.clone()),
                         },
                     );
                 }
@@ -498,6 +507,9 @@ impl ModuleInterface {
                     external_javascript: _,
                 }) => {
                     let mut id_map = IdMap::new();
+                    let (_, name) = name
+                        .as_ref()
+                        .expect("Function in a definition must be named");
                     let _ = functions.insert(
                         name.clone(),
                         FunctionInterface {
@@ -505,7 +517,7 @@ impl ModuleInterface {
                                 implementations,
                             ),
                             deprecation: DeprecationInterface::from_deprecation(deprecation),
-                            documentation: documentation.clone(),
+                            documentation: documentation.as_ref().map(|(_, doc)| doc.clone()),
                             parameters: arguments
                                 .iter()
                                 .map(|arg| ParameterInterface {
