@@ -1,10 +1,9 @@
-use crate::ast::SrcSpan;
+use crate::ast::{SrcSpan, TypeAst};
 use crate::error::wrap;
 use crate::parse::Token;
 use ecow::EcoString;
-use heck::{ToSnakeCase, ToUpperCamelCase};
 
-#[derive(Debug, PartialEq, Eq, Clone)]
+#[derive(Debug, PartialEq, Eq, Clone, Copy)]
 pub struct LexicalError {
     pub error: LexicalErrorType,
     pub location: SrcSpan,
@@ -18,7 +17,7 @@ pub enum InvalidUnicodeEscapeError {
     InvalidCodepoint,             // Invalid Unicode codepoint
 }
 
-#[derive(Debug, PartialEq, Eq, Clone)]
+#[derive(Debug, PartialEq, Eq, Clone, Copy)]
 pub enum LexicalErrorType {
     BadStringEscape,                                 // string contains an unescaped slash
     InvalidUnicodeEscape(InvalidUnicodeEscapeError), // \u{...} escape sequence is invalid
@@ -27,9 +26,6 @@ pub enum LexicalErrorType {
     RadixIntNoValue,                                 // 0x, 0b, 0o without a value
     UnexpectedStringEnd,                             // Unterminated string literal
     UnrecognizedToken { tok: char },
-    BadName { name: String },
-    BadDiscardName { name: String },
-    BadUpname { name: String },
     InvalidTripleEqual,
 }
 
@@ -261,6 +257,41 @@ utf16_codepoint, utf32_codepoint, signed, unsigned, big, little, native, size, u
                     "See: https://tour.gleam.run/flow-control/case-expressions/".into(),
                 ],
             ),
+            ParseErrorType::ExpectedRecordConstructor {
+                name,
+                public,
+                opaque,
+                field,
+                field_type,
+            } => {
+                let (accessor, opaque) = match *public {
+                    true if *opaque => ("pub ", "opaque "),
+                    true => ("pub ", ""),
+                    false => ("", ""),
+                };
+
+                let mut annotation = EcoString::new();
+                match field_type {
+                    Some(t) => t.print(&mut annotation),
+                    None => annotation.push_str("Type"),
+                };
+
+                (
+                    "I was not expecting this",
+                    vec![
+                        "Each custom type variant must have a constructor:\n".into(),
+                        format!("{accessor}{opaque}type {name} {{"),
+                        format!("  {name}("),
+                        format!("    {field}: {annotation},"),
+                        "  )".into(),
+                        "}".into(),
+                    ],
+                )
+            }
+            ParseErrorType::CallInClauseGuard => (
+                "Unsupported expression",
+                vec!["Functions cannot be called in clause guards.".into()],
+            ),
         }
     }
 }
@@ -319,6 +350,14 @@ pub enum ParseErrorType {
     RedundantInternalAttribute,          // for a private definition marked as internal
     InvalidModuleTypePattern,            // for patterns that have a dot like: `name.thing`
     ListPatternSpreadFollowedByElements, // When there is a pattern after a spread [..rest, pattern]
+    ExpectedRecordConstructor {
+        name: EcoString,
+        public: bool,
+        opaque: bool,
+        field: EcoString,
+        field_type: Option<TypeAst>,
+    },
+    CallInClauseGuard, // case x { _ if f() -> 1 }
 }
 
 impl LexicalError {
@@ -357,29 +396,6 @@ impl LexicalError {
                 "I can't figure out what to do with this character",
                 vec!["Hint: Is it a typo?".into()],
             ),
-            LexicalErrorType::BadName { name } => (
-                "This is not a valid name",
-                vec![
-                    "Hint: Names start with a lowercase letter and contain a-z, 0-9, or _."
-                        .to_string(),
-                    format!("Try: {}", name.to_snake_case()),
-                ],
-            ),
-            LexicalErrorType::BadDiscardName { name } => (
-                "This is not a valid discard name",
-                vec![
-                    "Hint: Discard names start with _ and contain a-z, 0-9, or _.".into(),
-                    format!("Try: _{}", name.to_snake_case()),
-                ],
-            ),
-            LexicalErrorType::BadUpname { name } => (
-                "This is not a valid upname",
-                vec![
-                    "Hint: Upnames start with an uppercase letter and contain".into(),
-                    "only lowercase letters, numbers, and uppercase letters.".into(),
-                    format!("Try: {}", name.to_upper_camel_case()),
-                ],
-            ),
             LexicalErrorType::InvalidUnicodeEscape(
                 InvalidUnicodeEscapeError::MissingOpeningBrace,
             ) => (
