@@ -7,6 +7,7 @@ use crate::parse::token::Token;
 use crate::warning::WarningEmitter;
 use camino::Utf8PathBuf;
 
+use ecow::EcoString;
 use itertools::Itertools;
 use pretty_assertions::assert_eq;
 
@@ -318,54 +319,6 @@ fn bit_array2() {
     );
 }
 
-#[test]
-fn name() {
-    assert_error!(
-        "let xS = 1",
-        ParseError {
-            error: ParseErrorType::LexError {
-                error: LexicalError {
-                    error: LexicalErrorType::BadName { name: "xS".into() },
-                    location: SrcSpan { start: 4, end: 6 },
-                }
-            },
-            location: SrcSpan { start: 4, end: 6 },
-        }
-    );
-}
-
-#[test]
-fn name1() {
-    assert_error!(
-        "let _xS = 1",
-        ParseError {
-            error: ParseErrorType::LexError {
-                error: LexicalError {
-                    error: LexicalErrorType::BadDiscardName { name: "_xS".into() },
-                    location: SrcSpan { start: 4, end: 7 },
-                }
-            },
-            location: SrcSpan { start: 4, end: 7 },
-        }
-    );
-}
-
-#[test]
-fn name2() {
-    assert_error!(
-        "type S_m = String",
-        ParseError {
-            error: ParseErrorType::LexError {
-                error: LexicalError {
-                    error: LexicalErrorType::BadUpname { name: "S_m".into() },
-                    location: SrcSpan { start: 5, end: 8 },
-                }
-            },
-            location: SrcSpan { start: 5, end: 8 },
-        }
-    );
-}
-
 // https://github.com/gleam-lang/gleam/issues/3125
 #[test]
 fn triple_equals() {
@@ -1036,6 +989,43 @@ type A {
     );
 }
 
+// Tests whether diagnostic presents an example of how to formulate a proper
+// record constructor based off a common user error pattern.
+// https://github.com/gleam-lang/gleam/issues/3324
+
+#[test]
+fn type_invalid_record_constructor() {
+    assert_module_error!(
+        "
+pub type User {
+    name: String,
+}
+"
+    );
+}
+
+#[test]
+fn type_invalid_record_constructor_without_field_type() {
+    assert_module_error!(
+        "
+pub opaque type User {
+    name
+}
+"
+    );
+}
+
+#[test]
+fn type_invalid_record_constructor_invalid_field_type() {
+    assert_module_error!(
+        r#"
+type User {
+    name: "Test User",
+}
+"#
+    );
+}
+
 #[test]
 fn type_invalid_type_name() {
     assert_module_error!(
@@ -1132,6 +1122,22 @@ const a = A(\"a\", let)
     );
 }
 
+// record access should parse even if there is no label written
+#[test]
+fn record_access_no_label() {
+    assert_parse_module!(
+        "
+type Wibble {
+    Wibble(wibble: String)
+}
+
+fn wobble() {
+  Wibble(\"a\").
+}
+"
+    );
+}
+
 #[test]
 fn newline_tokens() {
     assert_eq!(
@@ -1156,3 +1162,244 @@ case 2, 3 {
 }"
     );
 }
+
+#[test]
+fn const_string_concat() {
+    assert_parse_module!(
+        "
+const cute = \"cute\"
+const cute_bee = cute <> \"bee\"
+"
+    );
+}
+
+#[test]
+fn const_string_concat_naked_right() {
+    assert_module_error!(
+        "
+const no_cute_bee = \"cute\" <>
+"
+    );
+}
+
+#[test]
+fn function_call_in_case_clause_guard() {
+    assert_error!(
+        r#"
+let my_string = "hello"
+case my_string {
+    _ if length(my_string) > 2 -> io.debug("doesn't work')
+}"#
+    );
+}
+
+#[test]
+fn dot_access_function_call_in_case_clause_guard() {
+    assert_error!(
+        r#"
+let my_string = "hello"
+case my_string {
+    _ if string.length(my_string) > 2 -> io.debug("doesn't work')
+}"#
+    );
+}
+
+#[test]
+fn invalid_left_paren_in_case_clause_guard() {
+    assert_error!(
+        r#"
+let my_string = "hello"
+case my_string {
+    _ if string.length( > 2 -> io.debug("doesn't work')
+}"#
+    );
+}
+
+#[test]
+fn invalid_label_shorthand() {
+    assert_module_error!(
+        "
+pub fn main() {
+  wibble(:)
+}
+"
+    );
+}
+
+#[test]
+fn invalid_label_shorthand_2() {
+    assert_module_error!(
+        "
+pub fn main() {
+  wibble(:,)
+}
+"
+    );
+}
+
+#[test]
+fn invalid_label_shorthand_3() {
+    assert_module_error!(
+        "
+pub fn main() {
+  wibble(:arg)
+}
+"
+    );
+}
+
+#[test]
+fn invalid_label_shorthand_4() {
+    assert_module_error!(
+        "
+pub fn main() {
+  wibble(arg::)
+}
+"
+    );
+}
+
+#[test]
+fn invalid_label_shorthand_5() {
+    assert_module_error!(
+        "
+pub fn main() {
+  wibble(arg::arg)
+}
+"
+    );
+}
+
+#[test]
+fn invalid_pattern_label_shorthand() {
+    assert_module_error!(
+        "
+pub fn main() {
+  let Wibble(:) = todo
+}
+"
+    );
+}
+
+#[test]
+fn invalid_pattern_label_shorthand_2() {
+    assert_module_error!(
+        "
+pub fn main() {
+  let Wibble(:arg) = todo
+}
+"
+    );
+}
+
+#[test]
+fn invalid_pattern_label_shorthand_3() {
+    assert_module_error!(
+        "
+pub fn main() {
+  let Wibble(arg::) = todo
+}
+"
+    );
+}
+
+#[test]
+fn invalid_pattern_label_shorthand_4() {
+    assert_module_error!(
+        "
+pub fn main() {
+  let Wibble(arg: arg:) = todo
+}
+"
+    );
+}
+
+#[test]
+fn invalid_pattern_label_shorthand_5() {
+    assert_module_error!(
+        "
+pub fn main() {
+  let Wibble(arg1: arg2:) = todo
+}
+"
+    );
+}
+
+fn first_parsed_docstring(src: &str) -> EcoString {
+    let parsed =
+        crate::parse::parse_module(Utf8PathBuf::from("test/path"), src, &WarningEmitter::null())
+            .expect("should parse");
+
+    parsed
+        .module
+        .definitions
+        .first()
+        .expect("parsed a definition")
+        .definition
+        .get_doc()
+        .expect("definition without doc")
+}
+
+#[test]
+fn doc_comment_before_comment_is_not_attached_to_following_function() {
+    assert_eq!(
+        first_parsed_docstring(
+            r#"
+    /// Not included!
+    // pub fn call()
+
+    /// Doc!
+    pub fn wibble() {}
+"#
+        ),
+        " Doc!\n"
+    )
+}
+
+#[test]
+fn doc_comment_before_comment_is_not_attached_to_following_type() {
+    assert_eq!(
+        first_parsed_docstring(
+            r#"
+    /// Not included!
+    // pub fn call()
+
+    /// Doc!
+    pub type Wibble
+"#
+        ),
+        " Doc!\n"
+    )
+}
+
+#[test]
+fn doc_comment_before_comment_is_not_attached_to_following_type_alias() {
+    assert_eq!(
+        first_parsed_docstring(
+            r#"
+    /// Not included!
+    // pub fn call()
+
+    /// Doc!
+    pub type Wibble = Int
+"#
+        ),
+        " Doc!\n"
+    )
+}
+
+#[test]
+fn doc_comment_before_comment_is_not_attached_to_following_constant() {
+    assert_eq!(
+        first_parsed_docstring(
+            r#"
+    /// Not included!
+    // pub fn call()
+
+    /// Doc!
+    pub const wibble = 1
+"#
+        ),
+        " Doc!\n"
+    );
+}
