@@ -2,7 +2,8 @@ use self::expression::CallKind;
 
 use super::*;
 use crate::ast::{
-    Assignment, AssignmentKind, Statement, TypedAssignment, UntypedExpr, PIPE_VARIABLE,
+    Assignment, AssignmentKind, ImplicitCallArgOrigin, Statement, TypedAssignment, UntypedExpr,
+    PIPE_VARIABLE,
 };
 use vec1::Vec1;
 
@@ -98,14 +99,15 @@ impl<'a, 'b, 'c> PipeTyper<'a, 'b, 'c> {
                     ..
                 } => {
                     let fun = self.expr_typer.infer(*fun)?;
-                    match fun.type_().fn_arity() {
-                        // Rewrite as right(left, ..args)
-                        Some(arity) if arity == arguments.len() + 1 => {
-                            self.infer_insert_pipe(fun, arguments, location)?
-                        }
-
+                    match fun.type_().fn_types() {
                         // Rewrite as right(..args)(left)
-                        _ => self.infer_apply_to_call_pipe(fun, arguments, location)?,
+                        Some((args, return_))
+                            if args.len() == arguments.len() && return_.fn_arity() == Some(1) =>
+                        {
+                            self.infer_apply_to_call_pipe(fun, arguments, location)
+                        }
+                        // Rewrite as right(left, ..args)
+                        _ => self.infer_insert_pipe(fun, arguments, location),
                     }
                 }
 
@@ -132,7 +134,7 @@ impl<'a, 'b, 'c> PipeTyper<'a, 'b, 'c> {
             value: self.typed_left_hand_value_variable(),
             // This argument is given implicitly by the pipe, not explicitly by
             // the programmer.
-            implicit: true,
+            implicit: Some(ImplicitCallArgOrigin::Pipe),
         }
     }
 
@@ -145,7 +147,7 @@ impl<'a, 'b, 'c> PipeTyper<'a, 'b, 'c> {
             value: self.untyped_left_hand_value_variable(),
             // This argument is given implicitly by the pipe, not explicitly by
             // the programmer.
-            implicit: true,
+            implicit: Some(ImplicitCallArgOrigin::Pipe),
         }
     }
 
@@ -211,13 +213,13 @@ impl<'a, 'b, 'c> PipeTyper<'a, 'b, 'c> {
         function: TypedExpr,
         args: Vec<CallArg<UntypedExpr>>,
         location: SrcSpan,
-    ) -> Result<TypedExpr, Error> {
+    ) -> TypedExpr {
         let (function, args, typ) = self.expr_typer.do_infer_call_with_known_fun(
             function,
             args,
             location,
             CallKind::Function,
-        )?;
+        );
         let function = TypedExpr::Call {
             location,
             typ,
@@ -235,13 +237,13 @@ impl<'a, 'b, 'c> PipeTyper<'a, 'b, 'c> {
             args,
             location,
             CallKind::Function,
-        )?;
-        Ok(TypedExpr::Call {
+        );
+        TypedExpr::Call {
             location,
             typ,
             args,
             fun: Box::new(function),
-        })
+        }
     }
 
     /// Attempt to infer a |> b(c) as b(a, c)
@@ -250,7 +252,7 @@ impl<'a, 'b, 'c> PipeTyper<'a, 'b, 'c> {
         function: TypedExpr,
         mut arguments: Vec<CallArg<UntypedExpr>>,
         location: SrcSpan,
-    ) -> Result<TypedExpr, Error> {
+    ) -> TypedExpr {
         arguments.insert(0, self.untyped_left_hand_value_variable_call_argument());
         // TODO: use `.with_unify_error_situation(UnifyErrorSituation::PipeTypeMismatch)`
         // This will require the typing of the arguments to be lifted up out of
@@ -262,13 +264,13 @@ impl<'a, 'b, 'c> PipeTyper<'a, 'b, 'c> {
             arguments,
             location,
             CallKind::Function,
-        )?;
-        Ok(TypedExpr::Call {
+        );
+        TypedExpr::Call {
             location,
             typ,
             args,
             fun: Box::new(fun),
-        })
+        }
     }
 
     /// Attempt to infer a |> b as b(a)
@@ -281,6 +283,9 @@ impl<'a, 'b, 'c> PipeTyper<'a, 'b, 'c> {
             fn_(vec![self.argument_type.clone()], return_type.clone()),
         )
         .map_err(|e| {
+            if self.check_if_pipe_function_mismatch(&e) {
+                return convert_unify_error(flip_unify_error(e), function.location());
+            }
             let is_pipe_mismatch = self.check_if_pipe_type_mismatch(&e);
             let error = convert_unify_error(e, function.location());
             if is_pipe_mismatch {
@@ -317,6 +322,19 @@ impl<'a, 'b, 'c> PipeTyper<'a, 'b, 'c> {
         }
     }
 
+    fn check_if_pipe_function_mismatch(&mut self, error: &UnifyError) -> bool {
+        match error {
+            UnifyError::CouldNotUnify {
+                situation:
+                    Some(UnifyErrorSituation::FunctionsMismatch {
+                        reason: FunctionsMismatchReason::Arity { .. },
+                    }),
+                ..
+            } => true,
+            _ => false,
+        }
+    }
+
     fn warn_if_call_first_argument_is_hole(&mut self, call: &UntypedExpr) {
         if let UntypedExpr::Fn {
             is_capture: true,
@@ -326,13 +344,12 @@ impl<'a, 'b, 'c> PipeTyper<'a, 'b, 'c> {
         {
             if let Statement::Expression(UntypedExpr::Call { arguments, .. }) = body.first() {
                 match arguments.as_slice() {
-                    [first] | [first, ..] if first.is_capture_hole() => {
-                        self.expr_typer.environment.warnings.emit(
-                            Warning::RedundantPipeFunctionCapture {
-                                location: first.location,
-                            },
-                        )
-                    }
+                    [first] | [first, ..] if first.is_capture_hole() => self
+                        .expr_typer
+                        .problems
+                        .warning(Warning::RedundantPipeFunctionCapture {
+                            location: first.location,
+                        }),
                     _ => (),
                 }
             }
